package com.example.ueberwachungssystem;

import androidx.camera.core.ExperimentalGetImage;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ToggleButton;

import com.example.ueberwachungssystem.Fragments.Fragment1;
import com.example.ueberwachungssystem.Fragments.Fragment2;
import com.example.ueberwachungssystem.Detection.DetectorService;
import com.example.ueberwachungssystem.Fragments.Fragment3;
import com.example.ueberwachungssystem.Fragments.VideoListFragment;

@ExperimentalGetImage
public class MainActivity extends AppCompatActivity {
    //StringBuffer
    private StringBuffer messageBuffer = new StringBuffer();

    //Fragmente
    private Fragment aktuellesFragment;
    private Fragment1 fragment1;
    private Fragment2 fragment2;
    private Fragment3 fragment3;
    private VideoListFragment videoListFragment = new VideoListFragment();
    private DetectorService detectorService = new DetectorService();
    int num = 0;
    //Textviews
    private TextView Auswahl;
    private TextView AoderA;
    private String auswahltext = "Wahl des Detektionsmodus";
    private String auswahlAoderA = "Wahl von Alarmmeldungen oder Auswahl von Alarmaufzeichnungen";
    //Buttons
    private ToggleButton toggleKamera;
    private ToggleButton toggleAudio;
    private ToggleButton toggleBewegung;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setTitle(this.getClass().getSimpleName());
        setContentView(R.layout.activity_main);
        Auswahl = findViewById(R.id.textAuswahl);
        Auswahl.setText(auswahltext);
        AoderA = findViewById(R.id.textAoderA);
        AoderA.setText(auswahlAoderA);
        toggleKamera = findViewById(R.id.toggleKamera);
        toggleAudio = findViewById(R.id.toggleAudio);
        toggleBewegung = findViewById(R.id.toggleBewegung);
    }

    @Override
    protected void onResume() {
        super.onResume();

        PermissionHandler permissionHandler = new PermissionHandler(this);
        //permissionHandler.getPermissions();
        if (permissionHandler.hasPermissions()) {

            Intent serviceIntent = new Intent(this, DetectorService.class);
            bindService(serviceIntent, serviceConnection, Context.BIND_AUTO_CREATE);
            startService(serviceIntent);

            toggleKamera.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (toggleKamera.isChecked()) {
                        if(detectorService != null) {
                            detectorService.videoDetector.startDetection();
                        }
                    } else {
                        if(detectorService != null) {
                            detectorService.videoDetector.stopDetection();
                        }
                    }
                }
            });
            toggleAudio.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (toggleAudio.isChecked()) {
                        if(detectorService != null) {
                            detectorService.audioDetector.startDetection();
                        }
                    } else {
                        if(detectorService != null) {
                            detectorService.audioDetector.stopDetection();
                        }
                    }
                }
            });
            toggleBewegung.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (toggleBewegung.isChecked()) {
                        if(detectorService != null) {
                            detectorService.motionDetector.startDetection();
                        }
                    } else {
                        if(detectorService != null) {
                            detectorService.motionDetector.stopDetection();
                        }
                    }
                }
            });
        }else{
            Toast.makeText(this,"Bitte Rechte geben", Toast.LENGTH_SHORT).show();
        }

    }

    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.options_menu, menu);
        return true;
    }

    public boolean onOptionsItemSelected(MenuItem item) {
        Toast.makeText(this, "Selected Item: " + item.getTitle(), Toast.LENGTH_SHORT).show();
        PopUpClass popUpClass;
        switch (item.getItemId()) {
            case R.id.Rechteverwaltung:
                popUpClass = new PopUpClass(MainActivity.this);
                popUpClass.showPopupWindow(toggleAudio);
                popUpClass.RechtePrüfen();
                return true;
            case R.id.Sensoren:
                popUpClass = new PopUpClass(MainActivity.this);
                popUpClass.showPopupWindow(toggleAudio);
                popUpClass.Sensoren();
                return true;
            case R.id.Impressum:
                popUpClass = new PopUpClass(MainActivity.this);
                popUpClass.showPopupWindow(toggleAudio);
                popUpClass.Impressum();
                return true;
            case R.id.Detection:
                popUpClass = new PopUpClass(MainActivity.this);
                popUpClass.showPopupWindow(toggleAudio);
                popUpClass.DetectionTotal(num);
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    private ServiceConnection serviceConnection = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            DetectorService.ServiceBinder binder = (DetectorService.ServiceBinder) service;
            detectorService = binder.getBoundService();
//            detectorService.videoDetector.debugProcessing(null, outputImageView); //inputImageView

            detectorService.setOnDetectionListener(new DetectorService.OnDetectionListener() {
                @Override
                public void onDetection(@NonNull StringBuffer stringBuffer) {
                    Log.d("onDetection", stringBuffer.toString()); //Für oli hier Textview einbauen
                    num++;
                    messageBuffer = stringBuffer;
                    zeigeFragment(fragment1.erstellen(messageBuffer.toString()));
                }
            });
        }
        @Override
        public void onServiceDisconnected(ComponentName name) {
        }
    };

    public void onClickZeigeFragment1(View view) {
        Button button = (Button) view;
        zeigeFragment(fragment1.erstellen(messageBuffer.toString()));
    }

    public void onClickZeigeFragment2(View view) {
        Button button = (Button) view;
        //zeigeFragment(fragment2.erstellen("Hier stehen dann die Videos"));
        zeigeFragment(videoListFragment);
    }
    public void onClickZeigeFragment3(View view) {
        Button button = (Button) view;
    }

    public void onClickEntferneFragment(View view) {
        entferneFragment();
    }

    private void zeigeFragment(Fragment fragment) {
        FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
        ft.replace(R.id.frame, fragment);
        ft.commit();
        aktuellesFragment = fragment;
    }
    private void entferneFragment() {
        if (aktuellesFragment != null) {
            FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
            ft.remove(aktuellesFragment);
            ft.commit();
            aktuellesFragment = null ;
        }
    }
}